@@ -2,7 +2,6 @@ package AI::Genetic::Pro::Crossover::OX;
 
 use warnings;
 use strict;
-use Clone qw( clone );
 use List::MoreUtils qw(first_index);
 #use Data::Dumper; $Data::Dumper::Sortkeys = 1;
 #=======================================================================
@@ -47,7 +46,7 @@ sub run {
 					0;
 					
 						} map { 
-							clone($chromosomes->[$_])
+							$chromosomes->[$_]->clone;
 								} @elders;
 		
 		
